/**
 * Dns.h
 *
 * Copyright 2002, 2004 by Anthony Howe. All rights reserved.
 */

#ifndef __com_snert_lib_io_Dns_h__
#define __com_snert_lib_io_Dns_h__	1

#ifndef __com_snert_lib_type_Vector_h__
#include <com/snert/lib/type/Vector.h>
#endif

/*
 * Provide IPPROTO_* definitions.
 */
#if defined(__WIN32__)
#	if defined(__VISUALC__)
#		define _WINSOCK2API_
#	endif

#	include <windows.h>
#	include <winsock2.h>

#	define ETIMEDOUT	WSAETIMEDOUT
#else
#	include <sys/types.h>
#	include <netinet/in.h>
#endif /* __WIN32__ */

typedef void *Dns;

typedef struct DnsRecord {
	char *name;
	char *value;
	struct in_addr ip;
	unsigned long ttl;
	unsigned short type;
	unsigned short preference;
} *DnsRecord;

#ifdef __cplusplus
extern "C" {
#endif

extern int DnsInit(void);
extern Dns DnsOpen(const char **servers);
extern void DnsSetAttempts(Dns dns, int n);
extern void DnsSetTimeout(Dns dns, long ms);
extern void DnsClose(Dns dns);

/*
 * @param dns
 *	Dns object returned from DnsOpen(). 
 *
 * @param domain
 *	The domain, host name to lookup. An IP address returns itself.
 *
 * @return
 *	A Vector of DnsRecord objects.
 */
extern Vector DnsGetAddresses(Dns dns, const char *domain);
extern Vector DnsGetMailServers(Dns dns, const char *domain);
extern Vector DnsGetNameServers(Dns dns, const char *domain);

#define RCODE_NULL		-2
#define RCODE_ERRNO		-1
#define RCODE_OK		0
#define RCODE_FORMAT		1
#define RCODE_SERVER		2
#define RCODE_UNDEFINED		3
#define RCODE_NOT_IMPLEMENTED	4
#define RCODE_REFUSED		5

extern int DnsGetReturnCode(Dns dns);
extern const char *DnsGetError(Dns dns);
extern void DnsRecordRemoveEntry(void *);

#define DNS_DEBUG_ALL		(~0)
#define DNS_DEBUG_ENTER_EXIT	1
#define DNS_DEBUG_GET_SET	2
#define DNS_DEBUG_TRACE		8

extern void DnsSetDebugMask(int mask);

#ifdef  __cplusplus
}
#endif

#endif /* __com_snert_lib_io_Dns_h__ */
