/*
 * Time.h
 *
 * 	ANSI C		seconds 	10^1
 *	BSD, SYSTEMV	micro-seconds 	10^-6
 *	POSIX		nano-seconds	10^-9
 *
 * Copyright 2002, 2004 by Anthony Howe.  All rights reserved.
 */

#ifndef __com_snert_lib_sys_Time_h__
#define __com_snert_lib_sys_Time_h__	1

#define UNIT_ONE	1
#define UNIT_MILLI 	1000
#define UNIT_MICRO 	1000000
#define UNIT_NANO 	1000000000

/*
 * Redefined version of "struct timespec". See POSIX.1.
 */
typedef struct {
	long s;
	long ns;	/* 0 <= ns < 1 000 000 000 */
} Time;

#ifdef __cplusplus
extern "C" {
#endif

extern Time *TimeGetNow();
extern void TimeSetNow(Time *t);

extern Time *TimeClone(Time *t);
extern void TimeDestroy(Time *t);

extern void TimeAdd(Time *acc, Time *val);
extern void TimeSub(Time *acc, Time *val);

extern int TimeIsZero(Time *acc);

/*
 * Add an RFC 2821 style timestamp string to buffer. 
 * Return number bytes added or zero on error.
 */
extern long TimeStampAdd(char *buffer, long bufferLength);


#ifdef  __cplusplus
}
#endif

#endif /* __com_snert_lib_sys_Time_h__ */
