/*
 * Buf.h
 *
 * Copyright 2001, 2004 by Anthony Howe. All rights reserved.
 */

#ifndef __com_snert_lib_util_Buf_h__
#define __com_snert_lib_util_Buf_h__	1

#ifndef BUFSIZ
#include <stdio.h>
#endif

typedef void *Buf;

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Create
 */
extern Buf BufCreate(long);
extern Buf BufFromString(const char *);
extern Buf BufFromBuf(Buf, long, long);
extern Buf BufFromBytes(unsigned char *, long, long);
extern void BufDestroy(Buf);

/*
 * Convert a Buf object into byte string.
 * The original Buf is destroyed in the process.
 */
extern unsigned char *BufAsBytes(Buf);

/*
 * Access fields.
 */
extern long BufLength(Buf);
extern long BufCapacity(Buf);
extern unsigned char *BufBytes(Buf);
extern void BufSetLength(Buf, long);

/*
 * Extract bytes.
 */
extern char *BufToString(Buf);
extern unsigned char BufGetByte(Buf, long);
extern unsigned char *BufGetBytes(Buf, long, long);

/*
 * Replace bytes.
 */
extern void BufSetByte(Buf, long, unsigned char);
extern void BufSetBytes(Buf, long, unsigned char *, long, long);

/*
 * Compare
 */
extern int BufCompare(Buf, Buf);
extern int BufCompareBuf(Buf, long, Buf, long, long);

/*
 * Append. Return 0 for success or -1 on error.
 */
extern int BufAddBuf(Buf, Buf, long, long);
extern int BufAddByte(Buf, int);
extern int BufAddBytes(Buf, unsigned char *, long, long);
extern int BufAddString(Buf, char *);
extern int BufAddReadLine(Buf, int, long);
extern int BufAddInputLine(Buf, FILE *, long);

extern int BufInsertBytes(Buf, long, unsigned char *, long, long);

/*
 * In-place modifications
 */
extern void BufReverse(Buf, long, long);
extern void BufToLower(Buf, long, long);
extern void BufToUpper(Buf, long, long);

#ifdef  __cplusplus
}
#endif

#endif /* __com_snert_lib_util_Buf_h__ */
