#
# Log.pm
#
# Copyright 2002 by Anthony Howe.  All rights reserved.
#

#######################################################################
#	Nothing to configure below this point.
#######################################################################

package Log;

use vars qw($AUTHOR $VERSION);

$AUTHOR = 'achowe@snert.com';
$VERSION = '1.0';

use Symbol;

sub new {
	my ($class, $file, $level) = @_;
	my $self = gensym();
	bless $self;

	unless (open($self, ">>$file")) {
		die("Failed to open log file '$file': $!\n");
	}

	# Set log file to be line buffered.
	select( (select($self), $| = 1)[0] );

	$self{logFile} = $file;
	$self{logLevel} = $level;
	($self{program}) =  ($0 =~ m!/?([^\\/:]+)$!);

	return $self;
}

sub DESTROY {
	$_[0]->close;
}

sub close {
	close($_[0]);
}

sub setLevel {
	my ($self, $level) = @_;
	$self{logLevel} = $level;
}

sub message {
	my $self = shift;
	my $level = shift;

	unless ($self{logLevel} < $level) {
		my @now = localtime(time());

		printf $self "%4d-%02d-%02d %02d:%02d:%02d %s:%04u %s %s\n",
			$now[5]+1900, $now[4]+1, $now[3], $now[2], $now[1], $now[0],
			$self{program}, $$, qw(ERROR WARN INFO DEBUG)[$level], join('', @_);
	}
}

sub error {
	my $self = shift;
	$self->message(0, @_, ": $!");
	print STDERR join('', @_, ": $!\n");
}

sub warn {
	my $self = shift;
	$self->message(1, @_);
}

sub info {
	my $self = shift;
	$self->message(2, @_);
}

sub debug {
	my $self = shift;
	$self->message(3, @_);
}

1;

#######################################################################
#	End
#######################################################################

__END__
