/*
 * flock.c
 *
 * Copyright 2004, 2004 by Anthony Howe.  All rights reserved.
 */

#include <com/snert/lib/version.h>

#if !defined(HAVE_FLOCK)

#include <errno.h>
#include <fcntl.h>
#include <com/snert/lib/io/posix.h>

int
flock(int fd, int lock)
{
# if defined(HAVE_FCNTL)
	struct flock lockf;
	
	switch (lock) {
	case LOCK_SH: lockf.l_type = F_RDLCK; break;
	case LOCK_EX: lockf.l_type = F_WRLCK; break;
	case LOCK_UN: lockf.l_type = F_UNLCK; break;
	}
	
	lockf.l_whence = SEEK_SET;
	lockf.l_start = 0;
	lockf.l_len = 0;		/* until end of file */
	lockf.l_pid = 0;

	for (errno = 0; fcntl(fd, (lock & LOCK_NB) ? F_SETLK : F_SETLKW, &lockf) != 0; ) {
		if (errno != EINTR)
			return -1;
	}

	return 0;
# elif defined(HAVE_LOCKF)
	switch (lock) {
	case LOCK_SH: return 0;
	case LOCK_EX: lock = F_TLOCK; break;
	case LOCK_UN: lock = F_ULOCK; break;
	}

	return lockf(fd, lock, 0);
# elif defined(__BORLANDC__)
	switch (lock) {
	case LOCK_SH: return 0;
	case LOCK_EX: lock = LK_NBLCK; break;
	case LOCK_UN: lock = LK_UNLCK; break;
	}

	return locking(fd, lock, 0);
# else
#  error "No supported file locking method.";
# endif	
}

#endif /* !defined(HAVE_FLOCK) */
