/*
 * stdopen.c
 *
 * Copyright 1994, 2004 by Anthony Howe.  All rights reserved.
 */

#include <com/snert/lib/version.h>

#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include <com/snert/lib/io/Error.h>
#include <com/snert/lib/io/posix.h>

/* Stupid BCC and GCC can't agree. */
#ifndef __BORLANDC__
extern FILE *fdopen _((int, const char *));
#endif

static char filemsg[] = "File \"%s\" ";

/*
 * Either open a file, or reopen a standard stream.  When file is null or
 * is equal to the filename "-", then the standard stream to reopen is
 * specified by the standard file descriptor, stdio.  The standard
 * file descriptor is copied and the standard stream is closed.
 * The copied file descriptor is then opened as either a text or binary
 * stream for read, write, or append, as per fopen() mode string.
 * When file is not null, open the file as per fopen() mode string.
 *
 * Return the FILE * of the opened stream or null is an error occured.
 * An error message is written to stderr in the case of an error.
 */
FILE *
stdopen(file, mode, stdio)
const char *file, *mode;
int stdio;
{
	FILE *fp;
	static char *std[] = { "(standard input)", "(standard output)" };

	if (file == 0 || (file[0] == '-' && file[1] == '\0')) {
		int fd;

		/* stdin and stdout are a text stream by default.
		 * We may need to reopen the file as a binary stream.
		 */
		if ((fd = dup(stdio)) < 0) {
			ErrorPrintLine(__FILE__, __LINE__, filemsg, std[stdio]);
			return (FILE *) 0;
		}
#ifdef __BORLANDC__
		/* DOS people NEVER get it right.  The handling of
		 * CRLF is done at the stream level, NEVER at the
		 * file descriptor level, which should always be
		 * binary.
		 */
		if (0 <= strchr(mode, 'b') && setmode(fd, O_BINARY) < 0) {
			ErrorPrintLine(__FILE__, __LINE__, filemsg, std[stdio]);
			return (FILE *) 0;
		}
#endif
               	(void) fclose(stdio == 1 ? stdin : stdout);

		if ((fp = fdopen(fd, (char *) mode)) == 0) {
			ErrorPrintLine(__FILE__, __LINE__, filemsg, std[stdio]);
			return (FILE *) 0;
		}
	} else if ((fp = fopen(file, mode)) == 0) {
		ErrorPrintLine(__FILE__, __LINE__, filemsg, file);
		return (FILE *) 0;
	}

	return fp;
}
