/*
 * climits.c
 *
 * Copyright 1994, 2003 by Anthony Howe.  All rights reserved.
 */

#include <stdio.h>

#undef _
#if defined(__STDC__) || defined(__BORLANDC__)
# define _(x)			x
#else
# define _(x)			()
#endif

typedef struct {
	char *symbol;
	int size;
	int (*func)();
} t_limits;

int nbits;

char _int[] = "#define %s\t%d\n";
char _uint[] = "#define %s\t%u\n";
char _long[] = "#define %s\t%ld\n";
char _ulong[] = "#define %s\t%lu\n";

int bits _((t_limits *));
int char_is_unsigned _((t_limits *));
int min_char _((t_limits *));
int min _((t_limits *));
int max_char _((t_limits *));
int max _((t_limits *));
int abs _((t_limits *));
int int_is_short _((t_limits *));
int ones_complement _((t_limits *));

int
bits(ptr)
t_limits *ptr;
{
	int count;
	unsigned char byte;

	byte = 1;
	count = 0;

	do {
		++count;
		byte <<= 1;
	} while (byte != 0);

	if (ptr != (t_limits *) 0)
		printf(_int, ptr->symbol, count);

	return count;
}

int
char_is_unsigned(ptr)
t_limits *ptr;
{
	char min;
	int value;

	min = (char) (1 << (nbits - 1));
	value = 0 < min;

	if (ptr != (t_limits *) 0)
		printf(_int, ptr->symbol, value);

	return value;
}

int
min_char(ptr)
t_limits *ptr;
{
	char c;

	if (char_is_unsigned((t_limits *) 0))
		c = 0;
	else
		c = (char) (1 << (nbits - 1));
	printf(_int, ptr->symbol, (int) c);

	return 1;
}

int
min(ptr)
t_limits *ptr;
{
	int c;
	long l;
	short s;

	switch (ptr->size) {
	case sizeof (char):
		c = (char) (1 << (nbits - 1));
		if (0 < c)
			c = 1 - c;
		printf(_int, ptr->symbol, c);
		break;
	case sizeof (short):
		s = (short) (1 << (sizeof (short) * nbits -1));
		printf(_int, ptr->symbol, (int) s);
		break;
	case sizeof (long):
		l = (long) (1 << (sizeof (long) * nbits -1));
		printf(_long, ptr->symbol, l);
		break;
	}

	return 1;
}

int
max_char(ptr)
t_limits *ptr;
{
	char c;

	if (char_is_unsigned((t_limits *) 0))
		c = ~(char) 0;
	else
		c = (char) ~(1 << (nbits - 1));
	printf(_int, ptr->symbol, (unsigned) c);

	return 1;
}

int
max(ptr)
t_limits *ptr;
{
	int c;
	long l;
	short s;

	switch (ptr->size) {
	case sizeof (char):
		c = (unsigned char) ~((char) 1 << (nbits - 1));
		printf(_int, ptr->symbol, c);
		break;
	case sizeof (short):
		s = (short) (1 << (sizeof (short) * nbits -1));
		printf(_int, ptr->symbol, (int) ~s);
		break;
	case sizeof (long):
		l = (long) (1 << (sizeof (long) * nbits -1));
		printf(_long, ptr->symbol, ~l);
		break;
	}

	return 1;
}

int
abs(ptr)
t_limits *ptr;
{
	unsigned char c;
	unsigned long l;
	unsigned short s;

	switch (ptr->size) {
	case sizeof (char):
		c = ~(unsigned char) 0;
		printf(_uint, ptr->symbol, (unsigned) c);
		break;
	case sizeof (short):
		s = ~(unsigned short) 0;
		printf(_uint, ptr->symbol, (unsigned) s);
		break;
	case sizeof (long):
		l = ~(unsigned long) 0;
		printf(_ulong, ptr->symbol, l);
		break;
	}

	return 1;
}

int
int_is_short(ptr)
t_limits *ptr;
{
	printf(_int, ptr->symbol, sizeof (int) == sizeof (short));

	return 1;
}

int
ones_complement(ptr)
t_limits *ptr;
{
	printf(_int, ptr->symbol, !(-1 & 1));

	return !(-1 & 1);
}

t_limits limits[] = {
	{ "ONES_COMPLEMENT", 0, ones_complement },
	{ "CHAR_IS_UNSIGNED", 0, char_is_unsigned },
	{ "INT_IS_SHORT", 0, int_is_short },
	{ "BITS_PER_BYTE", 0, bits },
	{ "MAX_SCHAR", sizeof (char), max },
	{ "MIN_SCHAR", sizeof (char), min },
	{ "MAX_CHAR", sizeof (char), max_char },
	{ "MIN_CHAR", sizeof (char), min_char },
	{ "MAX_SHORT", sizeof (short), max },
	{ "MIN_SHORT", sizeof (short), min },
	{ "MAX_LONG", sizeof (long), max },
	{ "MIN_LONG", sizeof (long), min },
	{ "MAX_UCHAR", sizeof (unsigned char), abs },
	{ "MAX_USHORT", sizeof (unsigned short), abs },
	{ "MAX_ULONG", sizeof (unsigned long), abs },
	{ (char *) 0, 0, 0 }
};

int
main(argc, argv)
int argc;
char **argv;
{
	t_limits *ptr;

	nbits = bits((t_limits *) 0);

	for (ptr = limits; ptr->symbol != (char *) 0; ++ptr)
		(void) (*ptr->func)(ptr);

	return 0;
}



